"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ControllerAuth = exports.NoAuth = exports.Auth = exports.controllerLevelAuthPropertyName = exports.noAuthRoutesPropertyName = exports.authRoutesPropertyName = void 0;
const routeAuthInfo_1 = require("../entities/routeAuthInfo");
const controllerAuthInfo_1 = require("../entities/controllerAuthInfo");
exports.authRoutesPropertyName = 'authRoutes';
exports.noAuthRoutesPropertyName = 'noAuthRoutes';
exports.controllerLevelAuthPropertyName = 'controllerAuth';
function checkAndInitializeAuthRoutes(target) {
    if (!target.hasOwnProperty(exports.authRoutesPropertyName)) {
        Object.defineProperty(target, exports.authRoutesPropertyName, {
            value: new Array()
        });
    }
}
function checkAndInitializeNoAuthRoutes(target) {
    if (!target.hasOwnProperty(exports.noAuthRoutesPropertyName)) {
        Object.defineProperty(target, exports.noAuthRoutesPropertyName, {
            value: new Array()
        });
    }
}
/**
 * Specifies that a controller method is to be authenticated and authorized.
 * @param authCallback Method to call while performing authentication and authorization on this route.
 * This callback is optional and if not provided, the callback provided in the `@ControllerAuth` decorator (if used) will be used,
 * otherwise, `bootstrap` function's callback will be used.
 */
function Auth(authCallback) {
    return (target, propertyKey) => {
        checkAndInitializeAuthRoutes(target);
        const methodRegisteredForNoAuth = target[exports.noAuthRoutesPropertyName] ? target[exports.noAuthRoutesPropertyName].find(r => r.method === propertyKey) : undefined;
        if (methodRegisteredForNoAuth) {
            throw new Error(`The method ${propertyKey} is already marked with @NoAuth. Usage of @Auth and @NoAuth decorators is mutually exclusive.`);
        }
        target[exports.authRoutesPropertyName].push(new routeAuthInfo_1.RouteAuthInfo(propertyKey, authCallback));
    };
}
exports.Auth = Auth;
/**
 * Specifies that a controller method is excluded from authentication and authorization.
 * Useful when the controller is marked with `@ControllerAuth` decorator.
 */
function NoAuth() {
    return (target, propertyKey) => {
        checkAndInitializeNoAuthRoutes(target);
        const methodRegisteredForAuth = target[exports.authRoutesPropertyName] ? target[exports.authRoutesPropertyName].find(r => r.method === propertyKey) : undefined;
        if (methodRegisteredForAuth) {
            throw new Error(`The method ${propertyKey} is already marked with @Auth. Usage of @Auth and @NoAuth decorators is mutually exclusive.`);
        }
        target[exports.noAuthRoutesPropertyName].push(new routeAuthInfo_1.RouteAuthInfo(propertyKey));
    };
}
exports.NoAuth = NoAuth;
/**
 * Specifies that all the routes/method of a controller are to be authenticated and authorized.
 * @param authCallback Method to call while performing authentication and authorization on the routes of this method.
 * This callback is optional and if not provided, the callback provided in `bootstrap` function will be used.
 */
function ControllerAuth(authCallback) {
    return (constructor) => {
        constructor.prototype.controllerAuth = new controllerAuthInfo_1.ControllerAuthInfo(authCallback);
        return constructor;
    };
}
exports.ControllerAuth = ControllerAuth;
// export function ControllerAuth(authCallback?: (req: Request, endpoint: Endpoint) => AuthResponse) {
//     console.log('Called ControllerAuth');
//     return (target: any) => {
//         console.log('Called ControllerAuth Callback');
//         if (!target.hasOwnProperty(controllerLevelAuthPropertyName)) {
//             console.log(`${controllerLevelAuthPropertyName} property not defined`);
//             // Object.defineProperty(target, controllerLevelAuthPropertyName, {
//             //     value: new ControllerAuthInfo(authCallback)
//             // });
//             target[controllerLevelAuthPropertyName] = new ControllerAuthInfo(authCallback);
//             const json = JSON.stringify(target[controllerLevelAuthPropertyName]);
//             console.log(`Defined property ${controllerLevelAuthPropertyName}. Value: ${json}`);
//         }
//     }
// }
